@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: CartDebug.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: CartDebug.cxx 2504 2012-05-27 19:27:55Z stephena $
 //============================================================================
 
 #include "bspf.hxx"
@@ -69,9 +69,11 @@ CartDebug::CartDebug(Debugger& dbg, Console& console, const OSystem& osystem)
 
   // Add settings for Distella
   DiStella::settings.gfx_format =
-    myOSystem.settings().getInt("gfxformat") == 16 ? kBASE_16 : kBASE_2;
+    myOSystem.settings().getInt("dis.gfxformat") == 16 ? kBASE_16 : kBASE_2;
   DiStella::settings.show_addresses =
-    myOSystem.settings().getBool("showaddr");
+    myOSystem.settings().getBool("dis.showaddr");
+  DiStella::settings.rflag = myOSystem.settings().getBool("dis.relocate");
+  DiStella::settings.fflag = true;  // Not currently configurable
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -278,6 +280,7 @@ bool CartDebug::fillDisassemblyList(BankInfo& info, bool resolvedata, uInt16 sea
   bool found = false;
 
   myDisassembly.list.clear();
+  myDisassembly.list.reserve(2048);
   myDisassembly.fieldwidth = 10 + myLabelLength;
   DiStella distella(*this, myDisassembly.list, info,
                     myDisLabels, myDisDirectives, resolvedata);
@@ -622,7 +625,7 @@ string CartDebug::loadSymbolFile(string file)
     file += ".sym";
 
   FilesystemNode node(file);
-  if(node.exists() && !node.isDirectory())
+  if(node.exists() && node.isFile())
   {
     ifstream in(node.getPath().c_str());
     if(!in.is_open())
@@ -695,7 +698,7 @@ string CartDebug::loadConfigFile(string file)
     }
   }
 
-  if(node.exists() && !node.isDirectory())
+  if(node.exists() && node.isFile())
   {
     ifstream in(node.getPath().c_str());
     if(!in.is_open())
@@ -810,7 +813,7 @@ string CartDebug::saveConfigFile(string file)
         myOSystem.romFile()).getParent().getPath() + name + ".cfg");
   }
 
-  if(!node.isDirectory())
+  if(node.isFile())
   {
     ofstream out(node.getPath().c_str());
     if(!out.is_open())
