@@ -8,13 +8,13 @@
 //  SS  SS   tt   ee      ll   ll  aa  aa
 //   SSSS     ttt  eeeee llll llll  aaaaa
 //
-// Copyright (c) 1995-2011 by Bradford W. Mott, Stephen Anthony
+// Copyright (c) 1995-2012 by Bradford W. Mott, Stephen Anthony
 // and the Stella Team
 //
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: RiotWidget.cxx 2199 2011-01-01 16:04:32Z stephena $
+// $Id: RiotWidget.cxx 2346 2012-01-08 19:58:55Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -30,55 +30,27 @@
 #include "ToggleBitWidget.hxx"
 #include "Widget.hxx"
 
+#include "NullControlWidget.hxx"
+#include "JoystickWidget.hxx"
+#include "PaddleWidget.hxx"
+#include "BoosterWidget.hxx"
+#include "DrivingWidget.hxx"
+#include "GenesisWidget.hxx"
+#include "KeyboardWidget.hxx"
+
 #include "RiotWidget.hxx"
 
-#define CREATE_IO_REGS(desc, bits, bitsID)                               \
+#define CREATE_IO_REGS(desc, bits, bitsID, editable)                     \
   t = new StaticTextWidget(boss, font, xpos, ypos+2, lwidth, fontHeight, \
                            desc, kTextAlignLeft);                        \
   xpos += t->getWidth() + 5;                                             \
   bits = new ToggleBitWidget(boss, font, xpos, ypos, 8, 1);              \
   bits->setTarget(this);                                                 \
   bits->setID(bitsID);                                                   \
-  addFocusWidget(bits);                                                  \
+  if(editable) addFocusWidget(bits); else bits->setEditable(false);      \
   xpos += bits->getWidth() + 5;                                          \
   bits->setList(off, on);
 
-#define CREATE_PORT_PINS(label, pins, pinsID)                      \
-  t = new StaticTextWidget(boss, font, xpos, ypos+2, 14*fontWidth, \
-                           fontHeight, label, kTextAlignLeft);     \
-  xpos += t->getWidth()/2 - 5;  ypos += t->getHeight() + 5;        \
-  pins[0] = new CheckboxWidget(boss, font, xpos, ypos, "",         \
-                               kCheckActionCmd);                   \
-  pins[0]->setID(pinsID);                                          \
-  pins[0]->setTarget(this);                                        \
-  addFocusWidget(pins[0]);                                         \
-  ypos += pins[0]->getHeight() * 2 + 10;                           \
-  pins[1] = new CheckboxWidget(boss, font, xpos, ypos, "",         \
-                               kCheckActionCmd);                   \
-  pins[1]->setID(pinsID);                                          \
-  pins[1]->setTarget(this);                                        \
-  addFocusWidget(pins[1]);                                         \
-  xpos -= pins[0]->getWidth() + 5;                                 \
-  ypos -= pins[0]->getHeight() + 5;                                \
-  pins[2] = new CheckboxWidget(boss, font, xpos, ypos, "",         \
-                               kCheckActionCmd);                   \
-  pins[2]->setID(pinsID);                                          \
-  pins[2]->setTarget(this);                                        \
-  addFocusWidget(pins[2]);                                         \
-  xpos += (pins[0]->getWidth() + 5) * 2;                           \
-  pins[3] = new CheckboxWidget(boss, font, xpos, ypos, "",         \
-                               kCheckActionCmd);                   \
-  pins[3]->setID(pinsID);                                          \
-  pins[3]->setTarget(this);                                        \
-  addFocusWidget(pins[3]);                                         \
-  xpos -= (pins[0]->getWidth() + 5) * 2;                           \
-  ypos = 20 + (pins[0]->getHeight() + 10) * 3;                     \
-  pins[4] = new CheckboxWidget(boss, font, xpos, ypos, "Fire",     \
-                               kCheckActionCmd);                   \
-  pins[4]->setID(pinsID);                                          \
-  pins[4]->setTarget(this);                                        \
-  addFocusWidget(pins[4]);
-
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& font,
                      int x, int y, int w, int h)
@@ -104,22 +76,28 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& font,
   }
 
   // SWCHA bits in 'poke' mode
-  CREATE_IO_REGS("SWCHA(W):", mySWCHAWriteBits, kSWCHABitsID);
+  CREATE_IO_REGS("SWCHA(W):", mySWCHAWriteBits, kSWCHABitsID, true);
   col = xpos + 20;  // remember this for adding widgets to the second column
 
   // SWACNT bits
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWACNT:", mySWACNTBits, kSWACNTBitsID);
+  CREATE_IO_REGS("SWACNT:", mySWACNTBits, kSWACNTBitsID, true);
 
   // SWCHA bits in 'peek' mode
   xpos = 10;  ypos += lineHeight + 5;
-  CREATE_IO_REGS("SWCHA(R):", mySWCHAReadBits, 0);
-  mySWCHAReadBits->setEditable(false);
+  CREATE_IO_REGS("SWCHA(R):", mySWCHAReadBits, 0, false);
+
+  // SWCHB bits in 'poke' mode
+  xpos = 10;  ypos += 2 * lineHeight;
+  CREATE_IO_REGS("SWCHB(W):", mySWCHBWriteBits, kSWCHBBitsID, true);
+
+  // SWBCNT bits
+  xpos = 10;  ypos += lineHeight + 5;
+  CREATE_IO_REGS("SWBCNT:", mySWBCNTBits, kSWBCNTBitsID, true);
 
   // SWCHB bits in 'peek' mode
-  xpos = 10;  ypos += 2 * lineHeight;
-  CREATE_IO_REGS("SWCHB:", mySWCHBBits, 0);
-  mySWCHBBits->setEditable(false);
+  xpos = 10;  ypos += lineHeight + 5;
+  CREATE_IO_REGS("SWCHB(R):", mySWCHBReadBits, 0, false);
 
   // Timer registers (R/W)
   const char* writeNames[] = { "TIM1T:", "TIM8T:", "TIM64T:", "TIM1024T:" };
@@ -147,18 +125,56 @@ RiotWidget::RiotWidget(GuiObject* boss, const GUI::Font& font,
   myTimRead = new DataGridWidget(boss, font, xpos, ypos, 1, 3, 8, 32, kBASE_16);
   myTimRead->setTarget(this);
   myTimRead->setEditable(false);
-  addFocusWidget(myTimRead);
 
-  // Controller port pins (for now, only the latched pins)
+  // Controller ports
+  const RiotDebug& riot = instance().debugger().riotDebug();
   xpos = col;  ypos = 10;
-  CREATE_PORT_PINS("P0 Controller:", myP0Pins, kP0PinsID);
-  xpos = col + font.getStringWidth("P0 Controller:") + 20;  ypos = 10;
-  CREATE_PORT_PINS("P1 Controller:", myP1Pins, kP1PinsID);
+  myLeftControl = addControlWidget(boss, font, xpos, ypos,
+      riot.controller(Controller::Left));
+  xpos += myLeftControl->getWidth() + 15;
+  myRightControl = addControlWidget(boss, font, xpos, ypos,
+      riot.controller(Controller::Right));
+
+  // TIA INPTx registers (R), left port
+  const char* contLeftReadNames[] = { "INPT0:", "INPT1:", "INPT4:" };
+  xpos = col;  ypos += myLeftControl->getHeight() + 2 * lineHeight;
+  for(int row = 0; row < 3; ++row)
+  {
+    new StaticTextWidget(boss, font, xpos, ypos + row*lineHeight + 2,
+                         6*fontWidth, fontHeight, contLeftReadNames[row], kTextAlignLeft);
+  }
+  xpos += 6*fontWidth + 5;
+  myLeftINPT = new DataGridWidget(boss, font, xpos, ypos, 1, 3, 2, 8, kBASE_16);
+  myLeftINPT->setTarget(this);
+  myLeftINPT->setEditable(false);
+
+  // TIA INPTx registers (R), right port
+  const char* contRightReadNames[] = { "INPT2:", "INPT3:", "INPT5:" };
+  xpos = col + myLeftControl->getWidth() + 15;
+  for(int row = 0; row < 3; ++row)
+  {
+    new StaticTextWidget(boss, font, xpos, ypos + row*lineHeight + 2,
+                         6*fontWidth, fontHeight, contRightReadNames[row], kTextAlignLeft);
+  }
+  xpos += 6*fontWidth + 5;
+  myRightINPT = new DataGridWidget(boss, font, xpos, ypos, 1, 3, 2, 8, kBASE_16);
+  myRightINPT->setTarget(this);
+  myRightINPT->setEditable(false);
+
+  // TIA INPTx VBLANK bits (D6-latch, D7-dump) (R)
+  xpos = col + 20;  ypos += myLeftINPT->getHeight() + lineHeight;
+  myINPTLatch = new CheckboxWidget(boss, font, xpos, ypos, "INPT latch (VBlank D6)");
+  myINPTLatch->setTarget(this);
+  myINPTLatch->setEditable(false);
+  ypos += lineHeight + 5;
+  myINPTDump = new CheckboxWidget(boss, font, xpos, ypos, "INPT dump to gnd (VBlank D7)");
+  myINPTDump->setTarget(this);
+  myINPTDump->setEditable(false);
 
   // PO & P1 difficulty switches
   int pwidth = font.getStringWidth("B/easy");
   lwidth = font.getStringWidth("P0 Diff: ");
-  xpos = col;  ypos += 3 * lineHeight;
+  xpos = col;  ypos += 2 * lineHeight;
   items.clear();
   items.push_back("B/easy", "b");
   items.push_back("A/hard", "a");
@@ -231,8 +247,36 @@ void RiotWidget::loadConfig()
   // Update the SWCHA register booleans (peek mode)
   IO_REGS_UPDATE(mySWCHAReadBits, swchaReadBits);
 
-  // Update the SWCHB register booleans
-  IO_REGS_UPDATE(mySWCHBBits, swchbBits);
+  // Update the SWCHB register booleans (poke mode)
+  IO_REGS_UPDATE(mySWCHBWriteBits, swchbWriteBits);
+
+  // Update the SWBCNT register booleans
+  IO_REGS_UPDATE(mySWBCNTBits, swbcntBits);
+
+  // Update the SWCHB register booleans (peek mode)
+  IO_REGS_UPDATE(mySWCHBReadBits, swchbReadBits);
+
+  // Update TIA INPTx registers
+  alist.clear();  vlist.clear();  changed.clear();
+  alist.push_back(0);  vlist.push_back(state.INPT0);
+    changed.push_back(state.INPT0 != oldstate.INPT0);
+  alist.push_back(1);  vlist.push_back(state.INPT1);
+    changed.push_back(state.INPT1 != oldstate.INPT1);
+  alist.push_back(4);  vlist.push_back(state.INPT4);
+    changed.push_back(state.INPT4 != oldstate.INPT4);
+  myLeftINPT->setList(alist, vlist, changed);
+  alist.clear();  vlist.clear();  changed.clear();
+  alist.push_back(2);  vlist.push_back(state.INPT2);
+    changed.push_back(state.INPT2 != oldstate.INPT2);
+  alist.push_back(3);  vlist.push_back(state.INPT3);
+    changed.push_back(state.INPT3 != oldstate.INPT3);
+  alist.push_back(5);  vlist.push_back(state.INPT5);
+    changed.push_back(state.INPT5 != oldstate.INPT5);
+  myRightINPT->setList(alist, vlist, changed);
+
+  // Update TIA VBLANK bits
+  myINPTLatch->setState(riot.vblank(6));
+  myINPTDump->setState(riot.vblank(7));
 
   // Update timer write registers
   alist.clear();  vlist.clear();  changed.clear();
@@ -256,26 +300,16 @@ void RiotWidget::loadConfig()
     changed.push_back(state.TIMCLKS != oldstate.TIMCLKS);
   myTimRead->setList(alist, vlist, changed);
 
-  // Update port pins
-  // We invert the booleans, since in the UI it makes more sense that
-  // if, for example, the 'up' checkbox is set, it means 'go up'
-  myP0Pins[0]->setState(!state.P0_PIN1);
-  myP0Pins[1]->setState(!state.P0_PIN2);
-  myP0Pins[2]->setState(!state.P0_PIN3);
-  myP0Pins[3]->setState(!state.P0_PIN4);
-  myP0Pins[4]->setState(!state.P0_PIN6);
-  myP1Pins[0]->setState(!state.P1_PIN1);
-  myP1Pins[1]->setState(!state.P1_PIN2);
-  myP1Pins[2]->setState(!state.P1_PIN3);
-  myP1Pins[3]->setState(!state.P1_PIN4);
-  myP1Pins[4]->setState(!state.P1_PIN6);
-
-  // Console switches (invert reset/select for same reason as the pins)
+  // Console switches (inverted, since 'selected' in the UI
+  // means 'grounded' in the system)
   myP0Diff->setSelected((int)riot.diffP0());
   myP1Diff->setSelected((int)riot.diffP1());
   myTVType->setSelected((int)riot.tvType());
   mySelect->setState(!riot.select());
   myReset->setState(!riot.reset());
+
+  myLeftControl->loadConfig();
+  myRightControl->loadConfig();
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -320,22 +354,20 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
           value = Debugger::get_bits((BoolArray&)mySWACNTBits->getState());
           riot.swacnt(value & 0xff);
           break;
+        case kSWCHBBitsID:
+          value = Debugger::get_bits((BoolArray&)mySWCHBWriteBits->getState());
+          riot.swchb(value & 0xff);
+          break;
+        case kSWBCNTBitsID:
+          value = Debugger::get_bits((BoolArray&)mySWBCNTBits->getState());
+          riot.swbcnt(value & 0xff);
+          break;
       }
       break;
 
     case kCheckActionCmd:
       switch(id)
       {
-        case kP0PinsID:
-          riot.setP0Pins(!myP0Pins[0]->getState(), !myP0Pins[1]->getState(),
-                         !myP0Pins[2]->getState(), !myP0Pins[3]->getState(),
-                         !myP0Pins[4]->getState());
-          break;
-        case kP1PinsID:
-          riot.setP1Pins(!myP1Pins[0]->getState(), !myP1Pins[1]->getState(),
-                         !myP1Pins[2]->getState(), !myP1Pins[3]->getState(),
-                         !myP1Pins[4]->getState());
-          break;
         case kSelectID:
           riot.select(!mySelect->getState());
           break;
@@ -358,3 +390,26 @@ void RiotWidget::handleCommand(CommandSender* sender, int cmd, int data, int id)
       break;
   }
 }
+
+// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+ControllerWidget* RiotWidget::addControlWidget(GuiObject* boss, const GUI::Font& font,
+        int x, int y, Controller& controller)
+{
+  switch(controller.type())
+  {
+    case Controller::Joystick:
+      return new JoystickWidget(boss, font, x, y, controller);
+    case Controller::Paddles:
+      return new PaddleWidget(boss, font, x, y, controller);
+    case Controller::BoosterGrip:
+      return new BoosterWidget(boss, font, x, y, controller);
+    case Controller::Driving:
+      return new DrivingWidget(boss, font, x, y, controller);
+    case Controller::Genesis:
+      return new GenesisWidget(boss, font, x, y, controller);
+    case Controller::Keyboard:
+      return new KeyboardWidget(boss, font, x, y, controller);
+    default:
+      return new NullControlWidget(boss, font, x, y, controller);
+  }
+}
