@@ -14,7 +14,7 @@
 // See the file "License.txt" for information on usage and redistribution of
 // this file, and for a DISCLAIMER OF ALL WARRANTIES.
 //
-// $Id: FSNodePOSIX.cxx 2318 2011-12-31 21:56:36Z stephena $
+// $Id: FSNodePOSIX.cxx 2528 2012-06-07 13:29:56Z stephena $
 //
 //   Based on code from ScummVM - Scumm Interpreter
 //   Copyright (C) 2002-2004 The ScummVM project
@@ -33,6 +33,7 @@
 #include <cassert>
 #include <cstdio>
 #include <cstring>
+#include <cstdlib>
 #include <unistd.h>
 
 #include <sstream>
@@ -53,8 +54,9 @@ class POSIXFilesystemNode : public AbstractFilesystemNode
     /**
      * Creates a POSIXFilesystemNode for a given path.
      *
-     * @param path String with the path the new node should point to.
-     * @param verify true if the isValid and isDirectory flags should be verified during the construction.
+     * @param path    String with the path the new node should point to.
+     * @param verify  true if the isValid and isDirectory/isFile flags should
+     *                be verified during the construction.
      */
     POSIXFilesystemNode(const string& path, bool verify);
 
@@ -64,6 +66,7 @@ class POSIXFilesystemNode : public AbstractFilesystemNode
     const string& getPath() const   { return _path; }
     string getRelativePath() const;
     bool isDirectory() const { return _isDirectory; }
+    bool isFile() const      { return _isFile;      }
     bool isReadable() const  { return access(_path.c_str(), R_OK) == 0; }
     bool isWritable() const  { return access(_path.c_str(), W_OK) == 0; }
 
@@ -74,11 +77,13 @@ class POSIXFilesystemNode : public AbstractFilesystemNode
     string _displayName;
     string _path;
     bool _isDirectory;
+    bool _isFile;
     bool _isValid;
 
   private:
     /**
-     * Tests and sets the _isValid and _isDirectory flags, using the stat() function.
+     * Tests and sets the _isValid and _isDirectory/_isFile flags,
+     * using the stat() function.
      */
     virtual void setFlags();
 };
@@ -114,7 +119,13 @@ void POSIXFilesystemNode::setFlags()
   struct stat st;
 
   _isValid = (0 == stat(_path.c_str(), &st));
-  _isDirectory = _isValid ? S_ISDIR(st.st_mode) : false;
+  if(_isValid)
+  {
+    _isDirectory = S_ISDIR(st.st_mode);
+    _isFile = S_ISREG(st.st_mode);
+  }
+  else
+    _isDirectory = _isFile = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -125,29 +136,36 @@ POSIXFilesystemNode::POSIXFilesystemNode()
   _displayName = _path;
   _isValid = true;
   _isDirectory = true;
+  _isFile = false;
 }
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 POSIXFilesystemNode::POSIXFilesystemNode(const string& p, bool verify)
 {
-  // Expand '~/' and './' to the value of the HOME env variable
-  if ( p.length() >= 2 && (p[0] == '~' || p[0] == '.') && p[1] == '/')
+  // Expand '~' to the HOME environment variable
+  _path = p;
+  size_t home_pos = _path.find_first_of("~");
+  if(home_pos != string::npos)
   {
-    const char *home = getenv("HOME");
+    const char* home = getenv("HOME");
 #ifdef MAXPATHLEN
     if (home != NULL && strlen(home) < MAXPATHLEN)
 #else // No MAXPATHLEN, as happens on Hurd
     if (home != NULL)
 #endif
     {
-      _path = home;
-      // Skip over the tilde/dot.  We know that p contains at least
-      // two chars, so this is safe:
-      _path += p.c_str() + 1;
+      _path.replace(home_pos, 1, home);
     }
   }
-  else
-    _path = p;
+
+  // Get absolute path  
+#ifdef MAXPATHLEN
+  char buf[MAXPATHLEN];
+#else // No MAXPATHLEN, as happens on Hurd
+  char buf[1024];
+#endif
+  if(realpath(_path.c_str(), buf))
+    _path = buf;
 
   _displayName = lastPathComponent(_path);
 
@@ -229,12 +247,18 @@ bool POSIXFilesystemNode::getChildren(AbstractFSList& myList, ListMode mode,
       {
         struct stat st;
         if (stat(entry._path.c_str(), &st) == 0)
+        {
           entry._isDirectory = S_ISDIR(st.st_mode);
+          entry._isFile = S_ISREG(st.st_mode);
+        }
         else
-          entry._isDirectory = false;
+          entry._isDirectory = entry._isFile = false;
       }
       else
+      {
         entry._isDirectory = (dp->d_type == DT_DIR);
+        entry._isFile = (dp->d_type == DT_REG);
+      }
     }
 #endif
 
@@ -244,7 +268,7 @@ bool POSIXFilesystemNode::getChildren(AbstractFSList& myList, ListMode mode,
       continue;
 
     // Honor the chosen mode
-    if ((mode == FilesystemNode::kListFilesOnly && entry._isDirectory) ||
+    if ((mode == FilesystemNode::kListFilesOnly && !entry._isFile) ||
         (mode == FilesystemNode::kListDirectoriesOnly && !entry._isDirectory))
       continue;
 
